/*
    This file is part of kal.
    Copyright (C) 1999  Frank W. Josellis

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/* calendar.c */

#include <stdio.h>
#include <stdlib.h>
#include "kal.h"


extern int shift, extra;
extern char *monthname[];


/*-------------------------------------------------------------------------*/
/* Function `days_of_week_short'                                           */
/* The days of week for European or American calendars                     */
/*-------------------------------------------------------------------------*/
char *days_of_week_short()
{
  static char *days;

  if(shift == 5) 
    days = "Mo Tu We Th Fr Sa Su";
  else 
    days = "Su Mo Tu We Th Fr Sa";
  
  return(days);
}

/*-------------------------------------------------------------------------*/
/* Function `void_string'                                                  */
/* Subroutine used to format the calendar                                  */
/*-------------------------------------------------------------------------*/
char *void_string(n)
     int n;
{
  int i;
  char *string;
  
  string = (char *) malloc((n+1) * sizeof(char));
  for(i = 0; i < n; i++) string[i] = ' ';
  string[n] = '\0';
  
  return(string);
}


/*-------------------------------------------------------------------------*/
/* Function `monthkal'                                                     */
/* Formats the calendar for a specified month                              */
/*-------------------------------------------------------------------------*/
void monthkal(month,day,year,min,max,color,out)
     int month, day, year, min, max, color; 
     FILE *out;
{
  int i, j, N, d, w, a[6][7], iso = (shift == 5) ? 1 : 0;
  char *name;
  
  /* Format the head */
  i = 1;
  j = year;
  while(j /= 10) i++;

  name = monthname[month - 1];
  N = 19 - strlen(name) - i;
  N = N%2 ? (N+1)/2 : N/2;
  fprintf(out,"%s%s %i\n", void_string(N), name, year);
  fprintf(out,"%s  Week\n",days_of_week_short());

  /* Format the  month matrix */
  N = 1;
  d = (daynum(month,1,year) + shift)%7;
  for(i = 0; i < 6; i++){
    for(j = 0; j < 7; j++){
      a[i][j] = N - d;
      N++;
    }
  }

  if(extra){
    if(year == 1752 && month == 9)
      for(i = 0; i < 6; i++)
	for(j = 0; j < 7; j++)
	  a[i][j] += (a[i][j] < 3) ? 0 : 11;
  }
  else {
    if(year == 1582 && month == 10)
      for(i = 0; i < 6; i++)
	for(j = 0; j < 7; j++)
	  a[i][j] += (a[i][j] < 5) ? 0 : 10;
  }
  
  N = mlength(month,year);
  for(i = 0; i < 6; i++){
    for(j = 0; j < 7; j++){
      if(a[i][j] < 1 || a[i][j] > N) fprintf(out,"   ");
      else
	if(a[i][j] == day) 
	  fprintf(out,"%s%2i%s ",MODE_R1(color),a[i][j],MODE_N(color));
	else 
	  if(a[i][j] >= min && a[i][j] <= max) 
	    fprintf(out,"%s%2i%s ",MODE_R0(color),a[i][j],MODE_N(color));
	  else
	    fprintf(out,"%2i ", a[i][j]);    
    }
    if(i == 0 || a[i][0] <= N){
      w = (i == 0) ?  a[i][6] : a[i][0];
      fprintf(out,"  %s%2i%s",MODE_G0(color),kal_week(month,w,year,iso),
	      MODE_N(color));
    }
    fprintf(out,"\n");
  }
  fprintf(out,"\n");
}


/*-------------------------------------------------------------------------*/
/* Function `yearkal_head'                                                 */
/* Header for a year's calendar                                            */
/*-------------------------------------------------------------------------*/
void yearkal_head(k,out)
     int k; 
     FILE *out;
{
  int l, N;
  char *name[3];

  for(l = 0; l < 3; l++){
    name[l] = monthname[3*k + l]; 

    N = 20 - strlen(name[l]);
    N = N%2 ? (N+1)/2 : N/2;
    fprintf(out,"%s%s%s   ",void_string(N),name[l],
	    void_string(20 - N - strlen(name[l]))); 
  }
  fprintf(out,"\n");
  for(l = 0; l < 3; l++)  
    fprintf(out,"%s   ", days_of_week_short());
  fprintf(out,"\n");
}

/*-------------------------------------------------------------------------*/
/* Function `yearkal'                                                      */
/* Formatted output of a year's calendar                                   */
/*-------------------------------------------------------------------------*/
void yearkal(year,out)
     int year; 
     FILE *out;
{
  int i, j, k, l, N, d, a[6][7][3];

  /* Year as title */
  i = 1;
  j = year;
  while(j /= 10) i++;
  N = 66 - i;
  N = N%2 ? (N + 1)/2 : N/2;
  fprintf(out,"%s%i\n\n", void_string(N), year);

  /* Format the block of month matrices with headers */
  for(k = 0; k < 4; k++){
    yearkal_head(k,out);

    for(l = 0; l < 3; l++){
      N = 1;
      d = (daynum(3*k + 1 + l,1,year) + shift)%7;
      for(i = 0; i < 6; i++){
	for(j = 0; j < 7; j++){
	    a[i][j][l] = N - d;
	    N++;
	}
      }
    }

    if(extra){
      if(year == 1752 && k == 2)
	for(i = 0; i < 6; i++)
	  for(j = 0; j < 7; j++)
	    a[i][j][2] += (a[i][j][2] < 3) ? 0 : 11;
    }
    else {
      if(year == 1582 && k == 3)
	for(i = 0; i < 6; i++)
	  for(j = 0; j < 7; j++)
	    a[i][j][0] += (a[i][j][0] < 5) ? 0 : 10;
    }

    for(i = 0; i < 6; i++){
      for(l = 0; l < 3; l++){
	N = mlength(3*k + 1 + l,year);
	for(j = 0; j < 7; j++){
	  if(a[i][j][l] < 1 || a[i][j][l] > N) fprintf(out,"   ");
	  else fprintf(out,"%2i ", a[i][j][l]);
	}
	fprintf(out,"  ");
      }
      fprintf(out,"\n");
    }
  }
  fprintf(out,"\n");
}
