/*
    This file is part of kal.
    Copyright (C) 1999  Frank W. Josellis

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/* days.c */

#include <stdio.h>
#include "kal.h"
#include "moon.h"

extern int shift, extra;
extern char *monthname[];

static char *days_of_week[] = {"Sunday","Monday","Tuesday","Wednesday",
			       "Thursday","Friday","Saturday"};


/*-------------------------------------------------------------------------*/
/* Function 'mlength'                                                      */
/* Determines the length of a month                                        */
/*-------------------------------------------------------------------------*/
int mlength(month,year)
     int month, year;
{
  int m;

  switch(month){
  case 2:
    if(year > 1600 + 100*extra && !(year%100) && (year/100)%4) 
      m = 28;
    else 
      m = (year%4 || year == 4) ? 28 : 29;
    break;
  case 4:
  case 6:
  case 9:
  case 11:
    m = 30;
    break;
  default:
    m = 31;
  }
  return(m);
}


/*-------------------------------------------------------------------------*/
/* Function `daynum'                                                       */
/* Computes the difference between a given date and                        */
/* Saturday, December 31 0                                                 */
/*-------------------------------------------------------------------------*/
int daynum(month,day,year)
     int month, day, year;
{
  int i, leapdays, daycounter, centuries = 0, periods = 0;

  if(year > 1600){
    centuries = (year - 1601)/100; 
    periods   = (year - 1601)/400; 
  }

  leapdays = (year - 1)/4 - centuries + periods - 1;
  if(leapdays == -1) leapdays = 0;
  if(extra && year > 1700) leapdays++;
  daycounter = 365*(year - 1) + leapdays;

  if(month > 1)
    for(i = 1; i < month; i++)  
      daycounter += mlength(i,year);
  
  if(extra)
    if(year < 1752 || (year == 1752 && month < 9) ||
       (year == 1752 && month == 9 && day <= 2))
      return(daycounter + day);
    else
      return(daycounter + day - 11);
  else
    if(year < 1582 || (year == 1582 && month < 10) ||
       (year == 1582 && month == 10 && day <= 4))
      return(daycounter + day);
    else
      return(daycounter + day - 10);  
}


/*-------------------------------------------------------------------------*/
/* Function `inv_daynum'                                                   */
/* The inverse of `daynum'                                                 */
/*-------------------------------------------------------------------------*/
void inv_daynum(d,month,day,year)
     int d, *month, *day, *year;
{
  *year = 1;
  while(d > daynum(12,31,*year)) (*year)++;

  d -= daynum(1,1,*year) - 1;
  if(extra)
    d += (*year == 1752 && d > 246) ? 11 : 0;    
  else 
    d += (*year == 1582 && d > 277) ? 10 : 0;    

  *month = 1;
  while(d > mlength(*month,*year)){
    d -= mlength(*month,*year);
    (*month)++;
  }

  *day = d;
}

/*-------------------------------------------------------------------------*/
/* Function `print_day'                                                    */
/* Print the name of the days of the week                                  */
/*-------------------------------------------------------------------------*/
void print_day(month,day,year,color,out)
     int month, day, year, color; 
     FILE *out;
{
  int d, iso = (shift == 5) ? 1 : 0;
 
 d = daynum(month,day,year);
  if((extra && d + JMIN > 2361221.5) || (!extra && d + JMIN > 2299160.5)) 
    fprintf(out,"Gregorian:\t");
  else 
    fprintf(out,"Julian:\t\t");
  fprintf(out,"%s%s %i %i%s  ",MODE_G1(color),monthname[month - 1],day,year,
	  MODE_N(color));
  fprintf(out,"%s",days_of_week[(d - 1)%7]);

  d -= daynum(1,1,year) - 1;
  if(extra)
    d += (year == 1752 && d > 246) ? 11 : 0; 
  else
    d += (year == 1582 && d > 277) ? 10 : 0; 

  fprintf(out,"  %i:%i", d, year);
  fprintf(out,"  Week %i\n", kal_week(month,day,year,iso));
}


/*-------------------------------------------------------------------------*/
/* Function `numeric'                                                      */
/* Converts date format numeric -> standard                                */
/*-------------------------------------------------------------------------*/
void numeric(num,year,color,out)
     int num, year, color; 
     FILE *out;
{
  int month = 1, iso = (shift == 5) ? 1 : 0;

  fprintf(out,"%i:%i  ", num, year);
  while(num > mlength(month,year)){
    num -= mlength(month,year);
    month++;
  }
  fprintf(out,"%s%s",MODE_G1(color),
	  days_of_week[(daynum(month,num,year) + 6)%7]);
  fprintf(out,"  %s %i %i%s", monthname[month - 1], num, year,MODE_N(color));
  fprintf(out,"  Week %i\n", kal_week(month,num,year,iso));
}

/*-------------------------------------------------------------------------*/
/* Function `days_plus'                                                    */
/* Compute date + days                                                     */
/*-------------------------------------------------------------------------*/
void days_plus(month,day,year,num,color,out)
     int month, day, year, num, color; 
     FILE *out;
{
  int d, m = 1, y = 1, n;

  d = daynum(month,day,year) + num;
  while(d > daynum(12,31,y)) y++;

  d -= daynum(1,1,y) - 1;
  if(extra)
    d += (y == 1752 && d > 246) ? 11 : 0;    
  else 
    d += (y == 1582 && d > 277) ? 10 : 0;    
  n = d;

  while(d > mlength(m,y)){
    d -= mlength(m,y);
    m++;
  }
  fprintf(out,"%s%s",MODE_G1(color),days_of_week[(daynum(m,d,y) - 1)%7]);
  fprintf(out,"  %s %i %i%s", monthname[m - 1], d, y, MODE_N(color));
  fprintf(out,"  %i:%i\n", n, y);
}


/*-------------------------------------------------------------------------*/
/* Function `fmt_days_between'                                             */
/* Format for days detween dates                                           */
/*-------------------------------------------------------------------------*/
void fmt_days_between(m1,d1,y1,m2,d2,y2,color,out)
     int m1, d1, y1, m2, d2, y2, color; 
     FILE *out;
{
  register int m = m2, y = y2, n1, n2, months, days, years;

  n1 = daynum(m1,d1,y1) - daynum(m1,1,y1);
  n2 = daynum(m2,d2,y2) - daynum(m2,1,y2);
  if(n2 < n1){
    days = n2 + daynum(m1,mlength(m1,y1),y1) - daynum(m1,d1,y1) + 1;
    m--;
    if(!m){ 
      y--;
      m = 12;
    }
  }
  else 
    days = n2 - n1;

  months = (m - m1 + 12)%12;
  if(m < m1) y--;
  years = y - y1;
  
  fprintf(out,"%s%i%s year%s, %s%i%s month%s, %s%i%s day%s\n",
	  MODE_G1(color),EPL2(years,MODE_N(color)),
	  MODE_G1(color),EPL2(months,MODE_N(color)),
	  MODE_G1(color),EPL2(days,MODE_N(color)));
}


/*-------------------------------------------------------------------------*/
/* Function `first_sunday'                                                 */
/* The date of the first sunday of a given year                            */
/*-------------------------------------------------------------------------*/
int first_sunday(year)
     int year;
{
  int c, d = daynum(1,1,year);
  
  c = (d - 1)%7 ? 8 : 1;
  d = c - (d - 1)%7;
  return(d);
}

/*-------------------------------------------------------------------------*/
/* Function `kal_week'                                                     */
/* The calendar week corresponding to a given date                         */
/*-------------------------------------------------------------------------*/
int kal_week(month,day,year,iso)
     int month, day, year, iso;
{
  int w, w0 = 52, s, t, d = daynum(month,day,year);

  s = first_sunday(year);
  t = daynum(1,s,year);

  if(iso){
    if(year > 1)
      w0 = (s > 3) ? 1 : (first_sunday(year - 1) > 3) ? 53 : 52;
    
    if(d > t){ 
      w = (d + 8 - (d - 1)%7 - t)/7;
      w -= (d - 1)%7 ? 0 : 1;
      w += (w0 == 1) ? 1 : 0;
      w = (w == 53 && first_sunday(year + 1) > 3) ? 1 : w;
      
      if(extra)
	w += (year == 1752 && d - daynum(12,31,1751) > 246) ? 1 : 0;
      else
	w += (year == 1582 && d - daynum(31,12,1581) > 277) ? 2 : 0;
    }
    else
      w = w0;
  } 
  else {
    w = (d >= t) ? (d + 8 - (d - 1)%7 - t)/7 : 0;
    
    if((extra && year == 1752 && d - daynum(12,31,1751) > 246) ||
       (!extra && year == 1582 && d - daynum(12,31,1581) > 277))
      w += 2;
  }
  return(w);
}

/*-------------------------------------------------------------------------*/
/* Function `iso_week'                                                     */
/* Displays ISO week                                                       */
/*-------------------------------------------------------------------------*/
void iso_week(week,year,color,out)
     int week, year, color; 
     FILE *out;
{
  int w1, s, d, month1, day1, year1, month2, day2, year2;

  s = first_sunday(year);
  w1 = kal_week(1,s,year,1);
  d = daynum(1,s,year);
  d = (w1 == 1) ? d - 6 : d + 1;
  d += 7 * (week - 1);
  
  inv_daynum(d,&month1,&day1,&year1);
  inv_daynum(d + 6,&month2,&day2,&year2);
  
  if(month1 != month2){
    monthkal(month1,0,year1,day1,mlength(month1,year1),color,out);
    monthkal(month2,0,year2,1,day2,color,out);
  }
  else
    monthkal(month1,0,year1,day1,day2,color,out);

  fprintf(out,"ISO week %i.%i begins on ",week,year);
  fprintf(out,"%s%s %i %i%s",MODE_G1(color),monthname[month1 - 1], day1, year1,
	  MODE_N(color));
  fprintf(out,"  %i:%i\n",d - daynum(1,1,year1) + 1,year1);
}
