/*
    This file is part of kal.
    Copyright (C) 1999  Frank W. Josellis

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/* kal.h */

#define VERSION "1.3.2"

#define SITES_SYS "/usr/local/lib/kal-1.3.2/kal-sites"

#define MORE "more"
#define LESS "less -r"
#define MOST "most"

#ifndef _FILTER
#define FILTER 0
#else
#define FILTER _FILTER
#endif

#ifdef _ANGLICAN
#define EXTRA 1
#else
#define EXTRA 0
#endif

#ifdef _EUROPEAN_STYLE
#define SHIFT  5
#define SUNDAY 6
#else
#define SHIFT  6
#define SUNDAY 0
#endif

#ifdef _NO_COLORS
#define COLOR 0
#else
#define COLOR 1
#endif

/* ANSI Color modes */
#define MODE_R0(x) ((x)) ? "\x1B[31m" : ""
#define MODE_R1(x) ((x)) ? "\x1B[1;31m" : "" 
#define MODE_G0(x) ((x)) ? "\x1B[32m" : ""
#define MODE_G1(x) ((x)) ? "\x1B[1;32m" : ""
#define MODE_B1(x) ((x)) ? "\x1B[1;34m" : ""
#define MODE_N(x)  ((x)) ? "\x1B[0m" : "" 


#define JMIN 1721424.5  /* Julian date of 0:00:00 GMT January 1 1     */
#define JMAX 5373484.5  /* Julian date of 0:00:00 GMT January 1 10000 */

#define HEBREW_EPOCH  347995.5
#define HEBREW_MAX   5373779.5

#define ISLAMIC_EPOCH 1948438.5  /* Julian date of 0:00:00 July 15 622 */
#define ISLAMIC_MAX   5373393.5  /* Julian date of 0:00:00 Oct 2 9999  */


#ifdef __GNUC__
/* Function prototypes in days.c */
int mlength(int, int);
int daynum(int, int, int);
void inv_daynum(int, int *, int *, int*);
void print_day(int, int, int, int, FILE *);
void numeric(int, int, int, FILE *);
void days_plus(int, int, int, int, int, FILE *);
void fmt_days_between(int, int, int, int, int, int, int, FILE*); 
int first_sunday(int);
int kal_week(int, int, int, int);
void iso_week(int, int, int, FILE *);

/* Function prototypes in calendar.c */
char *days_of_week_short(void);
char *void_string(int);
void monthkal(int, int, int, int, int, int, FILE *);
void yearkal_head(int, FILE *);
void yearkal(int, FILE *);

/* Function prototypes in kal2tex.c */
char *texdays(void);
void texkal(int, int, int, FILE *);
void monthkal2tex(int, int, FILE *);
void kal2tex(int, FILE *);

/* Function prototypes in kal2html.c */
void html_days_of_week(void);
void monthkal2html(int, int, int, FILE *);
void html_header(int, FILE *);
void kal2html(int, int, FILE *);

/* Function prototypes in easter.c */
void easter_sunday(int, int, FILE *);
void easter_sunday_greek(int, int, FILE *);

/* Function prototypes in hebrew.c */
int hebrew_year_months(int);
unsigned int hebrew_delay_1(int);
int hebrew_delay_2(int);
int hebrew_year_days(int);
int hebrew_month_days(int, int);
double hebrew2jd(int,int,int);
void jd2hebrew(double, int *, int *, int *);
void hebrew_fmt_days_between(int, int, int, int, int, int, int, FILE *);
int hebrew_day_of_week(int, int, int);
void hebrew_monthkal(int, int, int, int, FILE *);
void hebrew_yearkal_head(int, int, FILE *);
void hebrew_yearkal(int, FILE *);
void RoshHaShanah(int,int, FILE *);
void print_hebrew_day(double, int, FILE *);

/* Function prototypes in islamic.c */
int islamic_year_days(int);
int islamic_month_days(int, int);
double islamic2jd(int, int, int);
void jd2islamic(double jd, int *, int *, int *);
int islamic_day_of_week(int, int, int);
void islamic_monthkal(int, int, int, int, FILE *);
void islamic_yearkal_head(int, FILE *);
void islamic_yearkal(int, FILE *);
void print_islamic_day(double, int, FILE *);

#else
int mlength();
int daynum();
void inv_daynum();
void print_day();
void numeric();
void days_plus();
void fmt_days_between(); 
int first_sunday();
int kal_week();
void iso_week();
char *days_of_week_short();
char *void_string();
void monthkal();
void yearkal_head();
void yearkal();
char *texdays();
void texkal();
void monthkal2tex();
void kal2tex();
void html_days_of_week();
void monthkal2html();
void html_header();
void kal2html();
void easter_sunday();
void easter_sunday_greek();
int hebrew_year_months();
unsigned int hebrew_delay_1();
int hebrew_delay_2();
int hebrew_year_days();
int hebrew_month_days();
double hebrew2jd();
void jd2hebrew();
void hebrew_fmt_days_between();
int hebrew_day_of_week();
void hebrew_monthkal();
void hebrew_yearkal_head();
void hebrew_yearkal();
void RoshHaShanah();
void print_hebrew_day();
int islamic_year_days();
int islamic_month_days();
double islamic2jd();
void jd2islamic();
int islamic_day_of_week();
void islamic_monthkal();
void islamic_yearkal_head();
void islamic_yearkal();
void print_islamic_day();
#endif

