/*
    This file is part of kal.
    Copyright (C) 1999  Frank W. Josellis

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/* kal2html.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "kal.h"


/* Set color defaults */
char *bodybg   = "#666666";
char *bodyfg   = "#000000";
char *yearbg   = "#0099ff";
char *monthbg  = "#ffaa00";
char *daybg    = "#cccccc";
char *sundayfg = "#bb0000";
char *linkbg   = "#005599";
char *linkfg   = "#ffff00";
char *vlinkfg  = "#00bbcc";
char *alinkfg  = "#ff6666";
int tblborder  = 4;


extern int shift, sunday, extra;
extern char *prog;
extern char *monthname[];

char **days;


/*-------------------------------------------------------------------------*/
/* Function `html_days_of_week'                                            */
/* Set European or American style for HTML calendars                       */
/*-------------------------------------------------------------------------*/
void html_days_of_week()
{
  static char *eudays[] = {"Mo","Tu","We","Th","Fr","Sa","Su"};
  static char *usdays[] = {"Su","Mo","Tu","We","Th","Fr","Sa"};

  days = (shift == 5) ? eudays : usdays; 
}


/*-------------------------------------------------------------------------*/
/* Function `monthkal2html'                                                */
/* HTML formatted calendar for a specified month                           */
/*-------------------------------------------------------------------------*/
void monthkal2html(month,year,opt,out)
     int month, year, opt; 
     FILE *out;
{
  int i, j, d, N, a[6][7];
  char htmlfile[13], *link, *location, *date = NULL, *ptr[2];
  FILE *html;

  /* Set colors */
  if(getenv("KAL_BODYBG")) 
    bodybg = getenv("KAL_BODYBG");
  if(getenv("KAL_BODYFG")) 
    bodyfg = getenv("KAL_BODYFG");
  if(getenv("KAL_MONTHBG")) 
    monthbg = getenv("KAL_MONTHBG");
  if(getenv("KAL_DAYBG")) 
    daybg = getenv("KAL_DAYBG");
  if(getenv("KAL_SUNDAYFG")) 
    sundayfg = getenv("KAL_SUNDAYFG");
  if(getenv("KAL_LINKBG")) 
    linkbg = getenv("KAL_LINKBG");
  if(getenv("KAL_LINKFG")) 
    linkfg = getenv("KAL_LINKFG");
  if(getenv("KAL_VLINKFG")) 
    vlinkfg = getenv("KAL_VLINKFG");
  if(getenv("KAL_ALINKFG")) 
    alinkfg = getenv("KAL_ALINKFG");
  if(getenv("KAL_TBLBORDER"))
    tblborder = atoi(getenv("KAL_TBLBORDER"));

  sprintf(htmlfile,"%i-%02i.html",year,month);
  if(opt == 'p'){
    if((html = fopen(htmlfile,"w")) == NULL){
      if(out != stdout) pclose(out);
      fprintf(stderr,"%s: Permission denied: %s\n\n", prog, htmlfile);
      exit(1);
    }
  }
  else 
    html = stdout;

  link = getenv("KAL_LINK");
  location = getenv("KAL_LOCATION");
  if(link && location){
    date = (char *) calloc(9,sizeof(char));
  }
  
  /* Formatting section */ 
  if(opt == 'o')
    fprintf(html,"</PRE>\n");
  
  fprintf(html,"<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML//EN//3.2\">\n");
  fprintf(html,"<HTML>\n");
  fprintf(html,"<!--\nFile: %s\n", htmlfile);
  fprintf(html,"Creator: kal %s\n-->\n", VERSION);
  
  fprintf(html,"<BODY BGCOLOR=%s TEXT=%s LINK=%s VLINK=%s ALINK=%s>",
	  bodybg,bodyfg,linkfg,vlinkfg,alinkfg);
  fprintf(html,"\n<CENTER>\n<TABLE BORDER=%i BGCOLOR=%s>\n",
	  tblborder,daybg);

  /* Monthname and year */
  fprintf(html,"<TR>\n");
  fprintf(html,"    <TH COLSPAN=7 BGCOLOR=%s>%s %i</TH>\n",
	  monthbg, monthname[month - 1], year);
  fprintf(html,"</TR>\n");

  /* The days of the week */
  fprintf(html,"<TR>\n");
  for(j = 0; j < 7; j++){
    if(j == sunday){
      fprintf(html,"    <TH><FONT COLOR=%s>",sundayfg);
      fprintf(html,"%s</FONT></TH>\n",days[j]);
    }
    else
      fprintf(html,"    <TH>%s</TH>\n",days[j]);
  }
  
  /* The month matrix */
  N = 1;
  d = (daynum(month,1,year) + shift)%7;
  for(i = 0; i < 6; i++){
    for(j = 0; j < 7; j++){
      a[i][j] = N - d;
      N++;
    }
  }
  
  if(extra){
    if(year == 1752 && month == 9)
      for(i = 0; i < 6; i++)
	for(j = 0; j < 7; j++)
	  a[i][j] += (a[i][j] < 3) ? 0 : 11;
  }
  else {
    if(year == 1582 && month == 10)
      for(i = 0; i < 6; i++)
	for(j = 0; j < 7; j++)
	  a[i][j] += (a[i][j] < 5) ? 0 : 10;
  }
  
  N = mlength(month,year);
  for(i = 0; i < 6; i++){
    if(a[i][0] <= N)
      fprintf(html,"<TR>\n");
    for(j = 0; j < 7; j++){
      if(a[i][j] < 1 || a[i][j] > N){
	if(a[i][0] <= N)
	  fprintf(html,"    <TD ALIGN=RIGHT></TD>\n");
      }
      else{
	if(link && location)
	  sprintf(date,"%02i%02i%04i",month,a[i][j],year);
	fprintf(html,"    <TD ALIGN=RIGHT");
	if(link && location && (ptr[0] = strstr(link,date)))
	  fprintf(html," BGCOLOR=%s><FONT COLOR=%s>|_</FONT>",
		  linkbg,linkbg);
	else
	  fprintf(html,"><FONT COLOR=%s>|_</FONT>",daybg);
	if(link && location && (ptr[0] = strstr(link,date))){
	  ptr[1] = strstr(ptr[0]," ");
	  *ptr[1] = 0;
	  fprintf(html,"<A HREF=\"%s#%s\">",location,ptr[0]);
	  fprintf(html,"<B>%2i</B></A>", a[i][j]);
	  *ptr[1] = 32;
	}
	else {
	  if(j == sunday){
	    if(!link || !location) fprintf(html,"<B>");
	    fprintf(html,"<FONT COLOR=%s>%2i</FONT>",
		    sundayfg,a[i][j]);
	    if(!link || !location) fprintf(html,"</B>");
	  }
	  else
	    fprintf(html,"%2i", a[i][j]);
	}
	fprintf(html,"</TD>\n");
      }
    }
    if(a[i][0] <= N)
      fprintf(html,"</TR>\n");
  }
  fprintf(html,"</TABLE>\n\n");
  
  fprintf(html,"</CENTER>\n</BODY>\n</HTML>\n");
  
  if(opt == 'o')
    fprintf(html,"<PRE>\n");
  else {
    fclose(html);
    fprintf(out,"%s: HTML source code written to %s\n", prog, htmlfile);
  }
}


/*-------------------------------------------------------------------------*/
/* Function `html_header'                                                  */
/* Header for a HTML calendar                                              */
/*-------------------------------------------------------------------------*/
void html_header(k,html)
     int k; 
     FILE *html;
{
  int l, j;

  /* The month names */
  fprintf(html,"<TR>\n");
  for(l = 0; l < 3; l++){
    fprintf(html,"    <TH COLSPAN=7 BGCOLOR=%s>%s</TH>\n",
	    monthbg, monthname[3*k + l]);
    if(l < 2)
      fprintf(html,"    <TH WIDTH=20></TH>\n");
  }
  fprintf(html,"</TR>\n");

  /* The days of the week */
  fprintf(html,"<TR>\n");
  for(l = 0; l < 3; l++){
    for(j = 0; j < 7; j++){
      if(j == sunday){
	fprintf(html,"    <TH><FONT COLOR=%s>",sundayfg);
	fprintf(html,"%s</FONT></TH>\n",days[j]);
      }
      else
	fprintf(html,"    <TH>%s</TH>\n",days[j]);
    }
    
    if(l < 2)
      fprintf(html,"    <TH WIDTH=20></TH>\n");
  }
  fprintf(html,"</TR>\n");
}


/*-------------------------------------------------------------------------*/
/* Function `kal2html'                                                     */
/* HTML formatted calendar                                                 */
/*-------------------------------------------------------------------------*/
void kal2html(year,opt,out)
     int year, opt; 
     FILE *out;
{
  int i, j, k, l, N, d, a[6][7][3];
  char htmlfile[10], *link, *location, *date = NULL, *ptr[2];
  FILE *html;

  /* Set colors */
  if(getenv("KAL_BODYBG")) 
    bodybg = getenv("KAL_BODYBG");
  if(getenv("KAL_BODYFG")) 
    bodyfg = getenv("KAL_BODYFG");
  if(getenv("KAL_YEARBG")) 
    yearbg = getenv("KAL_YEARBG");
  if(getenv("KAL_MONTHBG")) 
    monthbg = getenv("KAL_MONTHBG");
  if(getenv("KAL_DAYBG")) 
    daybg = getenv("KAL_DAYBG");
  if(getenv("KAL_SUNDAYFG")) 
    sundayfg = getenv("KAL_SUNDAYFG");
  if(getenv("KAL_LINKBG")) 
    linkbg = getenv("KAL_LINKBG");
  if(getenv("KAL_LINKFG")) 
    linkfg = getenv("KAL_LINKFG");
  if(getenv("KAL_VLINKFG")) 
    vlinkfg = getenv("KAL_VLINKFG");
  if(getenv("KAL_ALINKFG")) 
    alinkfg = getenv("KAL_ALINKFG");
  if(getenv("KAL_TBLBORDER"))
    tblborder = atoi(getenv("KAL_TBLBORDER"));

  sprintf(htmlfile,"%i.html",year);
  if(opt == 'p'){
    if((html = fopen(htmlfile,"w")) == NULL){
      if(out != stdout) pclose(out);
      fprintf(stderr,"%s: Permission denied: %s\n\n", prog, htmlfile);
      exit(1);
    }
  }
  else 
    html = stdout;

  link = getenv("KAL_LINK");
  location = getenv("KAL_LOCATION");
  if(link && location){
    date = (char *) calloc(9,sizeof(char));
  }
  
  /* Formatting section */ 
  if(opt == 'o')
    fprintf(html,"</PRE>\n");
  fprintf(html,"<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML//EN//3.2\">\n");
  fprintf(html,"<HTML>\n");
  fprintf(html,"<!--\nFile: %s\n", htmlfile);
  fprintf(html,"Creator: kal %s\n-->\n", VERSION);
  
  fprintf(html,"<BODY BGCOLOR=%s TEXT=%s LINK=%s VLINK=%s ALINK=%s>",
	  bodybg,bodyfg,linkfg,vlinkfg,alinkfg);
  fprintf(html,"\n<CENTER>\n<TABLE BORDER=%i BGCOLOR=%s>\n<TR>\n",
	  tblborder,daybg);
  fprintf(html,"    <TH COLSPAN=23 BGCOLOR=%s><FONT SIZE=5>%i</FONT></TH>\n"
	  "</TR>\n",yearbg,year);

  /* The month matrices with their headers */
  for(k = 0; k < 4; k++){
    if(k > 0)
      fprintf(html,"<TABLE BORDER=%i BGCOLOR=%s>\n",tblborder,daybg);
    html_header(k,html); 
  
    for(l = 0; l < 3; l++){
      N = 1;
      d = (daynum(3*k + 1 + l,1,year) + shift)%7;
      for(i = 0; i < 6; i++){
	for(j = 0; j < 7; j++){
	  a[i][j][l] = N - d;
	  N++;
	}
      }
    }
  
    if(extra){
      if(year == 1752 && k == 2)
	for(i = 0; i < 6; i++)
	  for(j = 0; j < 7; j++)
	    a[i][j][2] += (a[i][j][2] < 3) ? 0 : 11;
    }
    else {
      if(year == 1582 && k == 3)
	for(i = 0; i < 6; i++)
	  for(j = 0; j < 7; j++)
	    a[i][j][0] += (a[i][j][0] < 5) ? 0 : 10;
    }

    for(i = 0; i < 6; i++){
      fprintf(html,"<TR>\n");
      for(l = 0; l < 3; l++){
	N = mlength(3*k + 1 + l,year);
	for(j = 0; j < 7; j++){
	  if(a[i][j][l] < 1 || a[i][j][l] > N) 
	    fprintf(html,"    <TD ALIGN=RIGHT></TD>\n");
	  else{
	    if(link && location)
	      sprintf(date,"%02i%02i%04i",3*k + 1 + l,a[i][j][l],year);
	    fprintf(html,"    <TD ALIGN=RIGHT");
	    if(link && location && (ptr[0] = strstr(link,date)))
	      fprintf(html," BGCOLOR=%s><FONT COLOR=%s>|_</FONT>",
		      linkbg,linkbg);
	    else
	      fprintf(html,"><FONT COLOR=%s>|_</FONT>",daybg);
	    if(link && location && (ptr[0] = strstr(link,date))){
	      ptr[1] = strstr(ptr[0]," ");
	      *ptr[1] = 0;
	      fprintf(html,"<A HREF=\"%s#%s\">",location,ptr[0]);
	      fprintf(html,"<B>%2i</B></A>", a[i][j][l]);
	      *ptr[1] = 32;
	    }
	    else {
	      if(j == sunday){
		if(!link || !location) fprintf(html,"<B>");
		fprintf(html,"<FONT COLOR=%s>%2i</FONT>",
			sundayfg,a[i][j][l]);
		if(!link || !location) fprintf(html,"</B>");
	      }
	      else
		fprintf(html,"%2i", a[i][j][l]);
	    }
	    fprintf(html,"</TD>\n");
	  }
	}
	if(l < 2)
	  fprintf(html,"    <TD WIDTH=20></TD>\n");
      }
      fprintf(html,"</TR>\n");
    }
    fprintf(html,"</TABLE>\n\n");
  }

  fprintf(html,"</CENTER>\n</BODY>\n</HTML>\n");
  
  if(opt == 'o')
    fprintf(html,"<PRE>\n");
  else {
    fclose(html);
    fprintf(out,"%s: HTML source code written to %s\n", prog, htmlfile);
  }
}
