/*
    This file is part of kal.
    Copyright (C) 1999  Frank W. Josellis

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/* kal2tex.c */

#include <stdio.h>
#include <stdlib.h>
#include "kal.h"
#include "latex.h"


extern int shift, sunday, extra;
extern char *prog;
extern char *monthname[];


/*-------------------------------------------------------------------------*/
/* Function `texdays'                                                      */
/* Set European or American style for TeX calendars                        */
/*-------------------------------------------------------------------------*/
char *texdays()
{
  static char *days;

  if(shift == 5) 
    days = "Mo & Tu & We & Th & Fr & Sa & {\\bf Su}";
  else 
    days = "{\\bf Su} & Mo & Tu & We & Th & Fr & Sa";
  
  return(days);
}



/*-------------------------------------------------------------------------*/
/* Function 'texkal'                                                       */
/* TeX format of a month's calendar                                        */
/*-------------------------------------------------------------------------*/
void texkal(month,year,flag,tex)
     int month, year, flag; 
     FILE *tex;
{
  int i, j, N, d, a[6][7];

  /* The month matrix */
  N = 1;
  d = (daynum(month,1,year) + shift)%7;
  for(i = 0; i < 6; i++){
    for(j = 0; j < 7; j++){
      a[i][j] = N - d;
      N++;
    }
  }

  if(extra){
    if(year == 1752 && month == 9)
      for(i = 0; i < 6; i++)
	for(j = 0; j < 7; j++)
	  a[i][j] += (a[i][j] < 3) ? 0 : 11;
  }
  else {
    if(year == 1582 && month == 10)
      for(i = 0; i < 6; i++)
	for(j = 0; j < 7; j++)
	  a[i][j] += (a[i][j] < 5) ? 0 : 10;
  }

  /* TeX format of a month matrix */
  fprintf(tex,"\\begin{minipage}[t]{6cm} \n"
	  "\\begin{tabular}{|rrrrrrr|}\\hline \n"
	  "\\multicolumn{7}{|l|}{\\rule[-3mm]{0mm}{9mm}\\sc\\Large %s",
	  monthname[month - 1]);
  if(flag) fprintf(tex," %i", year);
  fprintf(tex,"} \\\\\n\\hline\\hline \n"
	  "%s \\\\ \\hline \n",texdays());
  N = mlength(month,year);
  for(i = 0; i < 6; i++){
    for(j = 0; j < 7; j++){
      if(a[i][j] < 1 || a[i][j] > N)
	fprintf(tex,"     ");
      else
	if(j == sunday)
	  fprintf(tex," {\\bf %i} ", a[i][j]);
	else
	  fprintf(tex," %i ", a[i][j]);

      if(j == 6)
	fprintf(tex,"\\\\ \n");
      else
	fprintf(tex,"&");
    }
  }
  fprintf(tex,"\\hline\n\\end{tabular} \n\\end{minipage} ");
}


/*-------------------------------------------------------------------------*/
/* Function 'monthkal2tex'                                                 */
/* TeX calendar for a specified month                                      */
/*-------------------------------------------------------------------------*/
void monthkal2tex(month,year,out)
     int year; 
     FILE *out;
{
  char texfile[12];
  FILE *tex;

  sprintf(texfile,"%i-%02i.tex", year, month);
  if((tex = fopen(texfile,"w")) == NULL){
    if(out != stdout) pclose(out); 
    fprintf(stderr,"%s: Permission denied: %s\n\n", prog, texfile);
    exit(1);
  }
  
  /* TeX format of a month calendar */
  fprintf(tex,"\\documentclass[%s]{article}\n\n", PAPERSIZE);
  fprintf(tex,"\\setlength{\\topmargin}{%s}\n", TOPMARGIN);
  fprintf(tex,"\\setlength{\\oddsidemargin}{%s}\n", ODDSIDEMARGIN);
  fprintf(tex,"\\setlength{\\textwidth}{%s}\n", TEXTWIDTH);
  fprintf(tex,"\\setlength{\\textheight}{%s}\n\n", TEXTHEIGHT);
  fprintf(tex,"\\begin{document} \n\\thispagestyle{empty} \n\n");

  texkal(month,year,1,tex);
  
  fprintf(tex,"\n\n\\end{document}");
  fclose(tex);
  fprintf(out,"%s: LaTeX source code written to %s\n", prog, texfile);
}

/*-------------------------------------------------------------------------*/
/* Function 'kal2tex'                                                      */
/* TeX formatted calendar                                                  */
/*-------------------------------------------------------------------------*/
void kal2tex(year,out)
     int year; 
     FILE *out;
{
  int k, l;
  char texfile[9];
  FILE *tex;

  sprintf(texfile,"%i.tex", year);
  if((tex = fopen(texfile,"w")) == NULL){
    if(out != stdout) pclose(out); 
    fprintf(stderr,"%s: Permission denied: %s\n\n", prog, texfile);
    exit(1);
  }
  
  /* TeX format of a year's calendar */
  fprintf(tex,"\\documentclass[%s]{article}\n\n", PAPERSIZE);
  fprintf(tex,"\\setlength{\\topmargin}{%s}\n", TOPMARGIN);
  fprintf(tex,"\\setlength{\\oddsidemargin}{%s}\n", ODDSIDEMARGIN);
  fprintf(tex,"\\setlength{\\textwidth}{%s}\n", TEXTWIDTH);
  fprintf(tex,"\\setlength{\\textheight}{%s}\n\n", TEXTHEIGHT);
  fprintf(tex,"\\begin{document} \n\\thispagestyle{empty} \n\n"
	  "\\begin{center} \n{\\Huge %i} \n\\end{center} \n\n", year);

  for(k = 0; k < 4; k++){
    fprintf(tex,"\\vspace{1.5cm} \n\\begin{tabular}{ccc} \n");
    for(l = 0; l < 3; l++){
      texkal(3*k+l+1,year,0,tex);
      switch(l){
      case 0:
      case 1:
	fprintf(tex," & \n\n");
	break;
      case 2:
	fprintf(tex,"\n\n\\end{tabular} \n\n");
	break;
      }
    }
  }

  fprintf(tex,"\\end{document}");
  fclose(tex);
  fprintf(out,"%s: LaTeX source code written to %s\n", prog, texfile);
}
