/*
    This file is part of kal.
    Copyright (C) 1999  Frank W. Josellis

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/* 
   Adopted from xmoontool by John Walker.
   Most of the code in moon.c as well as the content of the header
   file moon.h has been extracted from xmoontool.c.  

   Xmoontool is in the public domain so that the conditions of the 
   GNU General Public License do not apply to the original code.
   Xmoontool can be downloaded from http://www.fourmilab.ch/

   See the comments in moon.c for further notice.  
*/

/* moon.h */

#include <math.h>
#include <time.h>

#define UTIME "GMT"                /* Universal time label */

#define TRUE  1
#define FALSE 0

#define EPL(x) (x), (x) == 1 ? "" : "s"
#define EPL2(x,y) (x), (y), (x) == 1 ? "" : "s"

#ifndef PI
#ifdef  M_PI 
#define PI M_PI
#else
#define PI 3.14159265358979323846
#endif
#endif

/*  Astronomical constants  */
#define epoch	    2444238.5	   /* 1980 January 0.0 */

/*  Constants defining the Sun's apparent orbit  */
#define elonge	    278.833540	   /* Ecliptic longitude of the Sun
				      at epoch 1980.0 */
#define elongp	    282.596403	   /* Ecliptic longitude of the Sun at
				      perigee */
#define eccent      0.016718       /* Eccentricity of Earth's orbit */
#define sunsmax     1.495985e8     /* Semi-major axis of Earth's orbit, km */
#define sunangsiz   0.533128       /* Sun's angular size, degrees, at
				      semi-major axis distance */

/*  Elements of the Moon's orbit, epoch 1980.0  */
#define mmlong      64.975464      /* Moon's mean lonigitude at the epoch */
#define mmlongp     349.383063	   /* Mean longitude of the perigee at the
				      epoch */
#define mlnode	    151.950429	   /* Mean longitude of the node at the
				      epoch */
#define minc        5.145396       /* Inclination of the Moon's orbit */
#define mecc        0.054900       /* Eccentricity of the Moon's orbit */
#define mangsiz     0.5181         /* Moon's angular size at distance a
				      from Earth */
#define msmax       384401.0       /* Semi-major axis of Moon's orbit in km */
#define mparallax   0.9507	   /* Parallax at distance a from Earth */
#define synmonth    29.53058868    /* Synodic month (new Moon to new Moon) */
#define lunatbase   2423436.0      /* Base date for E. W. Brown's numbered
				      series of lunations (1923 January 16) */

/*  Properties of the Earth  */
#define earthrad    6378.16	   /* Radius of Earth in kilometres */


/*  Handy mathematical functions  */
#define sgn(x) (((x) < 0) ? -1 : ((x) > 0 ? 1 : 0))	  /* Extract sign */
#define abs(x) ((x) < 0 ? (-(x)) : (x)) 		  /* Absolute val */
#define fixangle(a) ((a) - 360.0 * (floor((a) / 360.0)))  /* Fix angle	  */
#define torad(d) ((d) * (PI / 180.0))			  /* Deg->Rad	  */
#define todeg(d) ((d) * (180.0 / PI))			  /* Rad->Deg	  */
#define dsin(x) (sin(torad((x))))			  /* Sin from deg */
#define dcos(x) (cos(torad((x))))			  /* Cos from deg */


#ifdef __GNUC__
/* Function prototypes in moon.c */
void inv_julian(double, int *, int *, int *);
void fmt_phase_time(double, int, int, char *, char *, char *, FILE *);
int jdate(struct tm *);
double jtime(struct tm *);
void jyear(double, int *, int *, int *);
void jhms(double, int *, int *, int *);
double meanphase(double, double);
double truephase(double, double);
void phasehunt(double, double *);
/*double kepler(double, double);*/
double phase(double,double *,double *,double *,double *,double *,double *);
void moon(double, int, FILE *);

/* Function prototypes in riset.c */
double eccentric_anomaly(double, double);
void updown(double, int, FILE *);
double localmoon(double, char *, char *, char *);

#else
void inv_julian();
void fmt_phase_time();
int jdate();
double jtime();
void jyear();
void jhms();
double meanphase();
double truephase();
void phasehunt();
/*double kepler();*/
double phase();
void moon();

double eccentric_anomaly();
void updown();
double *localmoon(); 
#endif
